@@ -5,7 +5,7 @@
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
-from sqlalchemy import sql, schema, types, exceptions, pool
+from sqlalchemy import sql, schema, types, exc, pool
 from sqlalchemy.sql import compiler, expression
 from sqlalchemy.engine import default, base
 
@@ -22,7 +22,7 @@ class AcNumeric(types.Numeric):
             else:
                 return str(value)
         return process
-        
+
     def get_col_spec(self):
         return "NUMERIC"
 
@@ -37,7 +37,7 @@ class AcFloat(types.Float):
                 return str(value)
             return None
         return process
-        
+
 class AcInteger(types.Integer):
     def get_col_spec(self):
         return "INTEGER"
@@ -83,7 +83,7 @@ class AcUnicode(types.Unicode):
         return None
 
 class AcChar(types.CHAR):
-    def get_col_spec(self):        
+    def get_col_spec(self):
         return "TEXT" + (self.length and ("(%d)" % self.length) or "")
 
 class AcBinary(types.Binary):
@@ -100,7 +100,7 @@ class AcBoolean(types.Boolean):
                 return None
             return value and True or False
         return process
-        
+
     def bind_processor(self, dialect):
         def process(value):
             if value is True:
@@ -112,24 +112,15 @@ class AcBoolean(types.Boolean):
             else:
                 return value and True or False
         return process
-        
+
 class AcTimeStamp(types.TIMESTAMP):
     def get_col_spec(self):
         return "TIMESTAMP"
 
-def descriptor():
-    return {'name':'access',
-    'description':'Microsoft Access',
-    'arguments':[
-        ('user',"Database user name",None),
-        ('password',"Database password",None),
-        ('db',"Path to database file",None),
-    ]}
-
 class AccessExecutionContext(default.DefaultExecutionContext):
     def _has_implicit_sequence(self, column):
         if column.primary_key and column.autoincrement:
-            if isinstance(column.type, types.Integer) and not column.foreign_key:
+            if isinstance(column.type, types.Integer) and not column.foreign_keys:
                 if column.default is None or (isinstance(column.default, schema.Sequence) and \
                                               column.default.optional):
                     return True
@@ -176,7 +167,7 @@ class AccessDialect(default.DefaultDialect):
         types.CHAR: AcChar,
         types.TIMESTAMP: AcTimeStamp,
     }
-
+    name = 'access'
     supports_sane_rowcount = False
     supports_sane_multi_rowcount = False
 
@@ -202,7 +193,7 @@ class AccessDialect(default.DefaultDialect):
                 except pythoncom.com_error:
                     pass
             else:
-                raise exceptions.InvalidRequestError("Can't find a DB engine. Check http://support.microsoft.com/kb/239114 for details.")
+                raise exc.InvalidRequestError("Can't find a DB engine. Check http://support.microsoft.com/kb/239114 for details.")
 
         import pyodbc as module
         return module
@@ -218,9 +209,6 @@ class AccessDialect(default.DefaultDialect):
             connectors.append("PWD=%s" % opts.get("password", ""))
         return [[";".join(connectors)], {}]
 
-    def create_execution_context(self, *args, **kwargs):
-        return AccessExecutionContext(self, *args, **kwargs)
-
     def last_inserted_ids(self):
         return self.context.last_inserted_ids
 
@@ -236,7 +224,7 @@ class AccessDialect(default.DefaultDialect):
             c.execute(statement, parameters)
             self.context.rowcount = c.rowcount
         except Exception, e:
-            raise exceptions.DBAPIError.instance(statement, parameters, e)
+            raise exc.DBAPIError.instance(statement, parameters, e)
 
     def has_table(self, connection, tablename, schema=None):
         # This approach seems to be more reliable that using DAO
@@ -246,7 +234,7 @@ class AccessDialect(default.DefaultDialect):
         except Exception, e:
             return False
 
-    def reflecttable(self, connection, table, include_columns):        
+    def reflecttable(self, connection, table, include_columns):
         # This is defined in the function, as it relies on win32com constants,
         # that aren't imported until dbapi method is called
         if not hasattr(self, 'ischema_names'):
@@ -261,18 +249,19 @@ class AccessDialect(default.DefaultDialect):
                 const.dbMemo:       AcText,
                 const.dbBoolean:    AcBoolean,
                 const.dbText:       AcUnicode, # All Access strings are unicode
+                const.dbCurrency:   AcNumeric,
             }
-            
+
         # A fresh DAO connection is opened for each reflection
         # This is necessary, so we get the latest updates
         dtbs = daoEngine.OpenDatabase(connection.engine.url.database)
-        
+
         try:
             for tbl in dtbs.TableDefs:
                 if tbl.Name.lower() == table.name.lower():
                     break
             else:
-                raise exceptions.NoSuchTableError(table.name)
+                raise exc.NoSuchTableError(table.name)
 
             for col in tbl.Fields:
                 coltype = self.ischema_names[col.Type]
@@ -290,7 +279,7 @@ class AccessDialect(default.DefaultDialect):
                 elif default:
                     if col.Type == const.dbBoolean:
                         default = default == 'Yes' and '1' or '0'
-                    colargs['default'] = schema.PassiveDefault(sql.text(default))
+                    colargs['server_default'] = schema.DefaultClause(sql.text(default))
 
                 table.append_column(schema.Column(col.Name, coltype, **colargs))
 
@@ -316,14 +305,14 @@ class AccessDialect(default.DefaultDialect):
                             col.unique = idx.Unique
                     else:
                         pass # TBD: multi-column indexes
-                
+
 
             for fk in dtbs.Relations:
                 if fk.ForeignTable != table.name:
                     continue
                 scols = [c.ForeignName for c in fk.Fields]
                 rcols = ['%s.%s' % (fk.Table, c.Name) for c in fk.Fields]
-                table.append_constraint(schema.ForeignKeyConstraint(scols, rcols))
+                table.append_constraint(schema.ForeignKeyConstraint(scols, rcols, link_to_name=True))
 
         finally:
             dtbs.Close()
@@ -333,19 +322,33 @@ class AccessDialect(default.DefaultDialect):
         # This is necessary, so we get the latest updates
         dtbs = daoEngine.OpenDatabase(connection.engine.url.database)
 
-        names = [t.Name for t in dtbs.TableDefs if t.Name[:4] != "MSys" and t.Name[:4] <> "~TMP"]
+        names = [t.Name for t in dtbs.TableDefs if t.Name[:4] != "MSys" and t.Name[:4] != "~TMP"]
         dtbs.Close()
         return names
 
 
 class AccessCompiler(compiler.DefaultCompiler):
+    extract_map = compiler.DefaultCompiler.extract_map.copy()
+    extract_map.update ({
+            'month': 'm',
+            'day': 'd',
+            'year': 'yyyy',
+            'second': 's',
+            'hour': 'h',
+            'doy': 'y',
+            'minute': 'n',
+            'quarter': 'q',
+            'dow': 'w',
+            'week': 'ww'
+    })
+
     def visit_select_precolumns(self, select):
         """Access puts TOP, it's version of LIMIT here """
         s = select.distinct and "DISTINCT " or ""
         if select.limit:
             s += "TOP %s " % (select.limit)
         if select.offset:
-            raise exceptions.InvalidRequestError('Access does not support LIMIT with an offset')
+            raise exc.InvalidRequestError('Access does not support LIMIT with an offset')
         return s
 
     def limit_clause(self, select):
@@ -357,7 +360,7 @@ class AccessCompiler(compiler.DefaultCompiler):
         return binary.operator == '%' and 'mod' or binary.operator
 
     def label_select_column(self, select, column, asfrom):
-        if isinstance(column, expression._Function):
+        if isinstance(column, expression.Function):
             return column.label()
         else:
             return super(AccessCompiler, self).label_select_column(select, column, asfrom)
@@ -369,7 +372,7 @@ class AccessCompiler(compiler.DefaultCompiler):
     def visit_function(self, func):
         """Access function names differ from the ANSI SQL names; rewrite common ones"""
         func.name = self.function_rewrites.get(func.name, func.name)
-        super(AccessCompiler, self).visit_function(func)
+        return super(AccessCompiler, self).visit_function(func)
 
     def for_update_clause(self, select):
         """FOR UPDATE is not supported by Access; silently ignore"""
@@ -378,18 +381,26 @@ class AccessCompiler(compiler.DefaultCompiler):
     # Strip schema
     def visit_table(self, table, asfrom=False, **kwargs):
         if asfrom:
-            return self.preparer.quote(table, table.name)
+            return self.preparer.quote(table.name, table.quote)
         else:
             return ""
 
+    def visit_join(self, join, asfrom=False, **kwargs):
+        return (self.process(join.left, asfrom=True) + (join.isouter and " LEFT OUTER JOIN " or " INNER JOIN ") + \
+            self.process(join.right, asfrom=True) + " ON " + self.process(join.onclause))
+
+    def visit_extract(self, extract):
+        field = self.extract_map.get(extract.field, extract.field)
+        return 'DATEPART("%s", %s)' % (field, self.process(extract.expr))
+
 
 class AccessSchemaGenerator(compiler.SchemaGenerator):
     def get_column_specification(self, column, **kwargs):
-        colspec = self.preparer.format_column(column) + " " + column.type.dialect_impl(self.dialect, _for_ddl=column).get_col_spec()
+        colspec = self.preparer.format_column(column) + " " + column.type.dialect_impl(self.dialect).get_col_spec()
 
         # install a sequence if we have an implicit IDENTITY column
         if (not getattr(column.table, 'has_sequence', False)) and column.primary_key and \
-                column.autoincrement and isinstance(column.type, types.Integer) and not column.foreign_key:
+                column.autoincrement and isinstance(column.type, types.Integer) and not column.foreign_keys:
             if column.default is None or (isinstance(column.default, schema.Sequence) and column.default.optional):
                 column.sequence = schema.Sequence(column.name + '_seq')
 
@@ -429,3 +440,4 @@ dialect.schemagenerator = AccessSchemaGenerator
 dialect.schemadropper = AccessSchemaDropper
 dialect.preparer = AccessIdentifierPreparer
 dialect.defaultrunner = AccessDefaultRunner
+dialect.execution_ctx_cls = AccessExecutionContext
