@@ -1,5 +1,5 @@
 # firebird.py
-# Copyright (C) 2005, 2006, 2007, 2008 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008, 2009 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -78,6 +78,18 @@ connections are active, the following setting may alleviate the problem::
   # Force SA to use a single connection per thread
   dialect.poolclass = pool.SingletonThreadPool
 
+RETURNING support
+-----------------
+
+Firebird 2.0 supports returning a result set from inserts, and 2.1 extends
+that to deletes and updates.
+
+To use this pass the column/expression list to the ``firebird_returning``
+parameter when creating the queries::
+
+  raises = tbl.update(empl.c.sales > 100, values=dict(salary=empl.c.salary * 1.1),
+                      firebird_returning=[empl.c.id, empl.c.salary]).execute().fetchall()
+
 
 .. [#] Well, that is not the whole story, as the client may still ask
        a different (lower) dialect...
@@ -87,10 +99,9 @@ connections are active, the following setting may alleviate the problem::
 """
 
 
-import datetime
-import warnings
+import datetime, decimal, re
 
-from sqlalchemy import exceptions, schema, types as sqltypes, sql, util
+from sqlalchemy import exc, schema, types as sqltypes, sql, util
 from sqlalchemy.engine import base, default
 
 
@@ -98,14 +109,14 @@ _initialized_kb = False
 
 
 class FBNumeric(sqltypes.Numeric):
-    """Handle ``NUMERIC(precision,length)`` datatype."""
+    """Handle ``NUMERIC(precision,scale)`` datatype."""
 
     def get_col_spec(self):
         if self.precision is None:
             return "NUMERIC"
         else:
-            return "NUMERIC(%(precision)s, %(length)s)" % { 'precision': self.precision,
-                                                            'length' : self.length }
+            return "NUMERIC(%(precision)s, %(scale)s)" % { 'precision': self.precision,
+                                                            'scale' : self.scale }
 
     def bind_processor(self, dialect):
         return None
@@ -115,7 +126,7 @@ class FBNumeric(sqltypes.Numeric):
             return None
         else:
             def process(value):
-                if isinstance(value, util.decimal_type):
+                if isinstance(value, decimal.Decimal):
                     return float(value)
                 else:
                     return value
@@ -163,7 +174,7 @@ class FBDateTime(sqltypes.DateTime):
         return process
 
 
-class FBDate(sqltypes.DateTime):
+class FBDate(sqltypes.Date):
     """Handle ``DATE`` datatype."""
 
     def get_col_spec(self):
@@ -188,14 +199,20 @@ class FBString(sqltypes.String):
     """Handle ``VARCHAR(length)`` datatype."""
 
     def get_col_spec(self):
-        return "VARCHAR(%(length)s)" % {'length' : self.length}
+        if self.length:
+            return "VARCHAR(%(length)s)" % {'length' : self.length}
+        else:
+            return "BLOB SUB_TYPE 1"
 
 
 class FBChar(sqltypes.CHAR):
     """Handle ``CHAR(length)`` datatype."""
 
     def get_col_spec(self):
-        return "CHAR(%(length)s)" % {'length' : self.length}
+        if self.length:
+            return "CHAR(%(length)s)" % {'length' : self.length}
+        else:
+            return "BLOB SUB_TYPE 1"
 
 
 class FBBinary(sqltypes.Binary):
@@ -236,7 +253,7 @@ ischema_names = {
        'DATE': lambda r: FBDate(),
        'TIME': lambda r: FBTime(),
        'TEXT': lambda r: FBString(r['flen']),
-      'INT64': lambda r: FBNumeric(precision=r['fprec'], length=r['fscale'] * -1), # This generically handles NUMERIC()
+      'INT64': lambda r: FBNumeric(precision=r['fprec'], scale=r['fscale'] * -1), # This generically handles NUMERIC()
      'DOUBLE': lambda r: FBFloat(),
   'TIMESTAMP': lambda r: FBDateTime(),
     'VARYING': lambda r: FBString(r['flen']),
@@ -244,17 +261,7 @@ ischema_names = {
        'BLOB': lambda r: r['stype']==1 and FBText() or FBBinary()
       }
 
-
-def descriptor():
-    return {'name':'firebird',
-    'description':'Firebird',
-    'arguments':[
-        ('host', 'Host Server Name', None),
-        ('database', 'Database Name', None),
-        ('user', 'Username', None),
-        ('password', 'Password', None)
-    ]}
-
+RETURNING_KW_NAME = 'firebird_returning'
 
 class FBExecutionContext(default.DefaultExecutionContext):
     pass
@@ -262,7 +269,7 @@ class FBExecutionContext(default.DefaultExecutionContext):
 
 class FBDialect(default.DefaultDialect):
     """Firebird dialect"""
-
+    name = 'firebird'
     supports_sane_rowcount = False
     supports_sane_multi_rowcount = False
     max_identifier_length = 31
@@ -273,7 +280,7 @@ class FBDialect(default.DefaultDialect):
         default.DefaultDialect.__init__(self, **kwargs)
 
         self.type_conv = type_conv
-        self.concurrency_level= concurrency_level
+        self.concurrency_level = concurrency_level
 
     def dbapi(cls):
         import kinterbasdb
@@ -295,9 +302,6 @@ class FBDialect(default.DefaultDialect):
             self.dbapi.init(type_conv=type_conv, concurrency_level=concurrency_level)
         return ([], opts)
 
-    def create_execution_context(self, *args, **kwargs):
-        return FBExecutionContext(self, *args, **kwargs)
-
     def type_descriptor(self, typeobj):
         return sqltypes.adapt_type(typeobj, colspecs)
 
@@ -321,7 +325,7 @@ class FBDialect(default.DefaultDialect):
         version = fbconn.server_version
         m = match('\w+-V(\d+)\.(\d+)\.(\d+)\.(\d+) \w+ (\d+)\.(\d+)', version)
         if not m:
-            raise exceptions.AssertionError("Could not determine version from string '%s'" % version)
+            raise AssertionError("Could not determine version from string '%s'" % version)
         return tuple([int(x) for x in m.group(5, 6, 4)])
 
     def _normalize_name(self, name):
@@ -392,6 +396,10 @@ class FBDialect(default.DefaultDialect):
     def is_disconnect(self, e):
         if isinstance(e, self.dbapi.OperationalError):
             return 'Unable to complete network request to host' in str(e)
+        elif isinstance(e, self.dbapi.ProgrammingError):
+            msg = str(e)
+            return ('Invalid connection state' in msg or
+                    'Invalid cursor state' in msg)
         else:
             return False
 
@@ -433,12 +441,28 @@ class FBDialect(default.DefaultDialect):
         WHERE rc.rdb$constraint_type=? AND rc.rdb$relation_name=?
         ORDER BY se.rdb$index_name, se.rdb$field_position
         """
+        # Heuristic-query to determine the generator associated to a PK field
+        genqry = """
+        SELECT trigdep.rdb$depended_on_name AS fgenerator
+        FROM rdb$dependencies tabdep
+             JOIN rdb$dependencies trigdep ON (tabdep.rdb$dependent_name=trigdep.rdb$dependent_name
+                                               AND trigdep.rdb$depended_on_type=14
+                                               AND trigdep.rdb$dependent_type=2)
+             JOIN rdb$triggers trig ON (trig.rdb$trigger_name=tabdep.rdb$dependent_name)
+        WHERE tabdep.rdb$depended_on_name=?
+          AND tabdep.rdb$depended_on_type=0
+          AND trig.rdb$trigger_type=1
+          AND tabdep.rdb$field_name=?
+          AND (SELECT count(*)
+               FROM rdb$dependencies trigdep2
+               WHERE trigdep2.rdb$dependent_name = trigdep.rdb$dependent_name) = 2
+        """
 
         tablename = self._denormalize_name(table.name)
 
         # get primary key fields
         c = connection.execute(keyqry, ["PRIMARY KEY", tablename])
-        pkfields =[self._normalize_name(r['fname']) for r in c.fetchall()]
+        pkfields = [self._normalize_name(r['fname']) for r in c.fetchall()]
 
         # get all of the fields for this table
         c = connection.execute(tblqry, [tablename])
@@ -456,10 +480,11 @@ class FBDialect(default.DefaultDialect):
             args = [name]
 
             kw = {}
-            # get the data types and lengths
+            # get the data type
             coltype = ischema_names.get(row['ftype'].rstrip())
             if coltype is None:
-                warnings.warn(RuntimeWarning("Did not recognize type '%s' of column '%s'" % (str(row['ftype']), name)))
+                util.warn("Did not recognize type '%s' of column '%s'" %
+                          (str(row['ftype']), name))
                 coltype = sqltypes.NULLTYPE
             else:
                 coltype = coltype(row)
@@ -474,20 +499,32 @@ class FBDialect(default.DefaultDialect):
             # does it have a default value?
             if row['fdefault'] is not None:
                 # the value comes down as "DEFAULT 'value'"
+                assert row['fdefault'].upper().startswith('DEFAULT '), row
                 defvalue = row['fdefault'][8:]
-                args.append(schema.PassiveDefault(sql.text(defvalue)))
+                args.append(schema.DefaultClause(sql.text(defvalue)))
 
-            table.append_column(schema.Column(*args, **kw))
+            col = schema.Column(*args, **kw)
+            if kw['primary_key']:
+                # if the PK is a single field, try to see if its linked to
+                # a sequence thru a trigger
+                if len(pkfields)==1:
+                    genc = connection.execute(genqry, [tablename, row['fname']])
+                    genr = genc.fetchone()
+                    if genr is not None:
+                        col.sequence = schema.Sequence(self._normalize_name(genr['fgenerator']))
+
+            table.append_column(col)
 
         if not found_table:
-            raise exceptions.NoSuchTableError(table.name)
+            raise exc.NoSuchTableError(table.name)
 
         # get the foreign keys
         c = connection.execute(fkqry, ["FOREIGN KEY", tablename])
         fks = {}
         while True:
             row = c.fetchone()
-            if not row: break
+            if not row:
+                break
 
             cname = self._normalize_name(row['cname'])
             try:
@@ -501,8 +538,8 @@ class FBDialect(default.DefaultDialect):
             fk[0].append(fname)
             fk[1].append(refspec)
 
-        for name,value in fks.iteritems():
-            table.append_constraint(schema.ForeignKeyConstraint(value[0], value[1], name=name))
+        for name, value in fks.iteritems():
+            table.append_constraint(schema.ForeignKeyConstraint(value[0], value[1], name=name, link_to_name=True))
 
     def do_execute(self, cursor, statement, parameters, **kwargs):
         # kinterbase does not accept a None, but wants an empty list
@@ -518,6 +555,15 @@ class FBDialect(default.DefaultDialect):
         connection.commit(True)
 
 
+def _substring(s, start, length=None):
+    "Helper function to handle Firebird 2 SUBSTRING builtin"
+
+    if length is None:
+        return "SUBSTRING(%s FROM %s)" % (s, start)
+    else:
+        return "SUBSTRING(%s FROM %s FOR %s)" % (s, start, length)
+
+
 class FBCompiler(sql.compiler.DefaultCompiler):
     """Firebird specific idiosincrasies"""
 
@@ -535,6 +581,9 @@ class FBCompiler(sql.compiler.DefaultCompiler):
         else:
             return self.process(alias.original, **kwargs)
 
+    functions = sql.compiler.DefaultCompiler.functions.copy()
+    functions['substring'] = _substring
+
     def function_argspec(self, func):
         if func.clauses:
             return self.process(func.clause_expr)
@@ -579,13 +628,48 @@ class FBCompiler(sql.compiler.DefaultCompiler):
             return self.LENGTH_FUNCTION_NAME + '%(expr)s'
         return super(FBCompiler, self).function_string(func)
 
+    def _append_returning(self, text, stmt):
+        returning_cols = stmt.kwargs[RETURNING_KW_NAME]
+        def flatten_columnlist(collist):
+            for c in collist:
+                if isinstance(c, sql.expression.Selectable):
+                    for co in c.columns:
+                        yield co
+                else:
+                    yield c
+        columns = [self.process(c, within_columns_clause=True)
+                   for c in flatten_columnlist(returning_cols)]
+        text += ' RETURNING ' + ', '.join(columns)
+        return text
+
+    def visit_update(self, update_stmt):
+        text = super(FBCompiler, self).visit_update(update_stmt)
+        if RETURNING_KW_NAME in update_stmt.kwargs:
+            return self._append_returning(text, update_stmt)
+        else:
+            return text
+
+    def visit_insert(self, insert_stmt):
+        text = super(FBCompiler, self).visit_insert(insert_stmt)
+        if RETURNING_KW_NAME in insert_stmt.kwargs:
+            return self._append_returning(text, insert_stmt)
+        else:
+            return text
+
+    def visit_delete(self, delete_stmt):
+        text = super(FBCompiler, self).visit_delete(delete_stmt)
+        if RETURNING_KW_NAME in delete_stmt.kwargs:
+            return self._append_returning(text, delete_stmt)
+        else:
+            return text
+
 
 class FBSchemaGenerator(sql.compiler.SchemaGenerator):
     """Firebird syntactic idiosincrasies"""
 
     def get_column_specification(self, column, **kwargs):
         colspec = self.preparer.format_column(column)
-        colspec += " " + column.type.dialect_impl(self.dialect, _for_ddl=True).get_col_spec()
+        colspec += " " + column.type.dialect_impl(self.dialect).get_col_spec()
 
         default = self.get_column_default_string(column)
         if default is not None:
@@ -599,8 +683,9 @@ class FBSchemaGenerator(sql.compiler.SchemaGenerator):
     def visit_sequence(self, sequence):
         """Generate a ``CREATE GENERATOR`` statement for the sequence."""
 
-        self.append("CREATE GENERATOR %s" % self.preparer.format_sequence(sequence))
-        self.execute()
+        if not self.checkfirst or not self.dialect.has_sequence(self.connection, sequence.name):
+            self.append("CREATE GENERATOR %s" % self.preparer.format_sequence(sequence))
+            self.execute()
 
 
 class FBSchemaDropper(sql.compiler.SchemaDropper):
@@ -609,8 +694,9 @@ class FBSchemaDropper(sql.compiler.SchemaDropper):
     def visit_sequence(self, sequence):
         """Generate a ``DROP GENERATOR`` statement for the sequence."""
 
-        self.append("DROP GENERATOR %s" % self.preparer.format_sequence(sequence))
-        self.execute()
+        if not self.checkfirst or self.dialect.has_sequence(self.connection, sequence.name):
+            self.append("DROP GENERATOR %s" % self.preparer.format_sequence(sequence))
+            self.execute()
 
 
 class FBDefaultRunner(base.DefaultRunner):
@@ -623,7 +709,7 @@ class FBDefaultRunner(base.DefaultRunner):
             self.dialect.identifier_preparer.format_sequence(seq))
 
 
-RESERVED_WORDS = util.Set(
+RESERVED_WORDS = set(
     ["action", "active", "add", "admin", "after", "all", "alter", "and", "any",
      "as", "asc", "ascending", "at", "auto", "autoddl", "avg", "based", "basename",
      "base_name", "before", "begin", "between", "bigint", "blob", "blobedit", "buffer",
@@ -670,7 +756,7 @@ class FBIdentifierPreparer(sql.compiler.IdentifierPreparer):
     reserved_words = RESERVED_WORDS
 
     def __init__(self, dialect):
-        super(FBIdentifierPreparer,self).__init__(dialect, omit_schema=True)
+        super(FBIdentifierPreparer, self).__init__(dialect, omit_schema=True)
 
 
 dialect = FBDialect
@@ -679,3 +765,4 @@ dialect.schemagenerator = FBSchemaGenerator
 dialect.schemadropper = FBSchemaDropper
 dialect.defaultrunner = FBDefaultRunner
 dialect.preparer = FBIdentifierPreparer
+dialect.execution_ctx_cls = FBExecutionContext
