@@ -1,5 +1,5 @@
 # informix.py
-# Copyright (C) 2005,2006, 2007, 2008 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005,2006, 2007, 2008, 2009 Michael Bayer mike_mp@zzzcomputing.com
 #
 # coding: gbk
 #
@@ -8,7 +8,7 @@
 
 import datetime
 
-from sqlalchemy import sql, schema, exceptions, pool, util
+from sqlalchemy import sql, schema, exc, pool, util
 from sqlalchemy.sql import compiler
 from sqlalchemy.engine import default
 from sqlalchemy import types as sqltypes
@@ -45,7 +45,7 @@ class InfoNumeric(sqltypes.Numeric):
         if not self.precision:
             return 'NUMERIC'
         else:
-            return "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
+            return "NUMERIC(%(precision)s, %(scale)s)" % {'precision': self.precision, 'scale' : self.scale}
 
 class InfoInteger(sqltypes.Integer):
     def get_col_spec(self):
@@ -160,7 +160,7 @@ ischema_names = {
     1   : InfoSmallInteger, # SMALLINT
     2   : InfoInteger,      # INT
     3   : InfoNumeric,      # Float
-    3   : InfoNumeric,      # SmallFloat
+    4   : InfoNumeric,      # SmallFloat
     5   : InfoNumeric,      # DECIMAL
     6   : InfoInteger,      # Serial
     7   : InfoDate,         # DATE
@@ -178,14 +178,6 @@ ischema_names = {
     -1  : InfoText,         # CLOB
 }
 
-def descriptor():
-    return {'name':'informix',
-    'description':'Informix',
-    'arguments':[
-        ('dsn', 'Data Source Name', None),
-        ('user', 'Username', None),
-        ('password', 'Password', None)
-    ]}
 
 class InfoExecutionContext(default.DefaultExecutionContext):
     # cursor.sqlerrd
@@ -197,7 +189,7 @@ class InfoExecutionContext(default.DefaultExecutionContext):
     # 5 - rowid after insert
     def post_exec(self):
         if getattr(self.compiled, "isinsert", False) and self.last_inserted_ids() is None:
-            self._last_inserted_ids = [self.cursor.sqlerrd[1],]
+            self._last_inserted_ids = [self.cursor.sqlerrd[1]]
         elif hasattr( self.compiled , 'offset' ):
             self.cursor.offset( self.compiled.offset )
         super(InfoExecutionContext, self).post_exec()
@@ -206,11 +198,12 @@ class InfoExecutionContext(default.DefaultExecutionContext):
         return informix_cursor( self.connection.connection )
 
 class InfoDialect(default.DefaultDialect):
+    name = 'informix'
     default_paramstyle = 'qmark'
     # for informix 7.31
     max_identifier_length = 18
 
-    def __init__(self, use_ansi=True,**kwargs):
+    def __init__(self, use_ansi=True, **kwargs):
         self.use_ansi = use_ansi
         default.DefaultDialect.__init__(self, **kwargs)
 
@@ -244,19 +237,13 @@ class InfoDialect(default.DefaultDialect):
         else:
             opt = {}
 
-        return ([dsn,], opt )
-
-    def create_execution_context(self , *args, **kwargs):
-        return InfoExecutionContext(self, *args, **kwargs)
-
-    def oid_column_name(self,column):
-        return "rowid"
+        return ([dsn], opt)
 
     def table_names(self, connection, schema):
         s = "select tabname from systables"
         return [row[0] for row in connection.execute(s)]
 
-    def has_table(self, connection, table_name,schema=None):
+    def has_table(self, connection, table_name, schema=None):
         cursor = connection.execute("""select tabname from systables where tabname=?""", table_name.lower() )
         return bool( cursor.fetchone() is not None )
 
@@ -264,18 +251,18 @@ class InfoDialect(default.DefaultDialect):
         c = connection.execute ("select distinct OWNER from systables where tabname=?", table.name.lower() )
         rows = c.fetchall()
         if not rows :
-            raise exceptions.NoSuchTableError(table.name)
+            raise exc.NoSuchTableError(table.name)
         else:
-            if table.owner is not None:
-                if table.owner.lower() in [r[0] for r in rows]:
-                    owner = table.owner.lower()
+            if table.schema is not None:
+                if table.schema.lower() in [r[0] for r in rows]:
+                    owner = table.schema.lower()
                 else:
-                    raise exceptions.AssertionError("Specified owner %s does not own table %s"%(table.owner, table.name))
+                    raise AssertionError("Specified owner %s does not own table %s"%(table.schema, table.name))
             else:
                 if len(rows)==1:
                     owner = rows[0][0]
                 else:
-                    raise exceptions.AssertionError("There are multiple tables with name %s in the schema, you must specifie owner"%table.name)
+                    raise AssertionError("There are multiple tables with name %s in the schema, you must specifie owner"%table.name)
 
         c = connection.execute ("""select colname , coltype , collength , t3.default , t1.colno from syscolumns as t1 , systables as t2 , OUTER sysdefaults as t3
                                     where t1.tabid = t2.tabid and t2.tabname=? and t2.owner=?
@@ -284,7 +271,7 @@ class InfoDialect(default.DefaultDialect):
         rows = c.fetchall()
 
         if not rows:
-            raise exceptions.NoSuchTableError(table.name)
+            raise exc.NoSuchTableError(table.name)
 
         for name , colattr , collength , default , colno in rows:
             name = name.lower()
@@ -317,7 +304,7 @@ class InfoDialect(default.DefaultDialect):
 
             colargs = []
             if default is not None:
-                colargs.append(schema.PassiveDefault(sql.text(default)))
+                colargs.append(schema.DefaultClause(sql.text(default)))
 
             table.append_column(schema.Column(name, coltype, nullable = (nullable == 0), *colargs))
 
@@ -341,8 +328,8 @@ class InfoDialect(default.DefaultDialect):
             try:
                 fk = fks[cons_name]
             except KeyError:
-               fk = ([], [])
-               fks[cons_name] = fk
+                fk = ([], [])
+                fks[cons_name] = fk
             refspec = ".".join([remote_table, remote_column])
             schema.Table(remote_table, table.metadata, autoload=True, autoload_with=connection)
             if local_column not in fk[0]:
@@ -351,7 +338,7 @@ class InfoDialect(default.DefaultDialect):
                 fk[1].append(refspec)
 
         for name, value in fks.iteritems():
-            table.append_constraint(schema.ForeignKeyConstraint(value[0], value[1] , None ))
+            table.append_constraint(schema.ForeignKeyConstraint(value[0], value[1] , None, link_to_name=True ))
 
         # PK
         c = connection.execute("""select t1.constrname as cons_name , t1.constrtype as cons_type ,
@@ -403,7 +390,7 @@ class InfoCompiler(compiler.DefaultCompiler):
         # TODO: dont modify the original select, generate a new one
         a = [ __label(c) for c in select._raw_columns ]
         for c in select._order_by_clause.clauses:
-            if ( __label(c) not in a ) and getattr( c , 'name' , '' ) != 'oid':
+            if ( __label(c) not in a ):
                 select.append_column( c )
 
         return compiler.DefaultCompiler.visit_select(self, select)
@@ -421,12 +408,8 @@ class InfoCompiler(compiler.DefaultCompiler):
         else:
             return compiler.DefaultCompiler.visit_function( self , func )
 
-    def visit_clauselist(self, list):
-        try:
-            li = [ c for c in list.clauses if c.name != 'oid' ]
-        except:
-            li = [ c for c in list.clauses ]
-        return ', '.join([s for s in [self.process(c) for c in li] if s is not None])
+    def visit_clauselist(self, list, **kwargs):
+        return ', '.join([s for s in [self.process(c) for c in list.clauses] if s is not None])
 
 class InfoSchemaGenerator(compiler.SchemaGenerator):
     def get_column_specification(self, column, first_pk=False):
@@ -436,7 +419,7 @@ class InfoSchemaGenerator(compiler.SchemaGenerator):
             colspec += " SERIAL"
             self.has_serial = True
         else:
-            colspec += " " + column.type.dialect_impl(self.dialect, _for_ddl=column).get_col_spec()
+            colspec += " " + column.type.dialect_impl(self.dialect).get_col_spec()
             default = self.get_column_default_string(column)
             if default is not None:
                 colspec += " DEFAULT " + default
@@ -507,3 +490,4 @@ dialect.statement_compiler = InfoCompiler
 dialect.schemagenerator = InfoSchemaGenerator
 dialect.schemadropper = InfoSchemaDropper
 dialect.preparer = InfoIdentifierPreparer
+dialect.execution_ctx_cls = InfoExecutionContext
