@@ -24,7 +24,7 @@ Known issues / TODO:
 
 import datetime, operator
 
-from sqlalchemy import util, sql, schema, exceptions
+from sqlalchemy import util, sql, schema, exc
 from sqlalchemy.sql import compiler, expression
 from sqlalchemy.engine import default, base
 from sqlalchemy import types as sqltypes
@@ -46,7 +46,7 @@ __all__ = [
     ]
 
 
-RESERVED_WORDS = util.Set([
+RESERVED_WORDS = set([
     "add", "all", "alter", "and",
     "any", "as", "asc", "backup",
     "begin", "between", "bigint", "binary",
@@ -160,26 +160,26 @@ class SybaseTypeError(sqltypes.TypeEngine):
 
     def bind_processor(self, dialect):
         def process(value):
-            raise exceptions.NotSupportedError("Data type not supported", [value])
+            raise exc.InvalidRequestError("Data type not supported", [value])
         return process
 
     def get_col_spec(self):
-        raise exceptions.NotSupportedError("Data type not supported")
+        raise exc.CompileError("Data type not supported")
 
 class SybaseNumeric(sqltypes.Numeric):
     def get_col_spec(self):
-        if self.length is None:
+        if self.scale is None:
             if self.precision is None:
                 return "NUMERIC"
             else:
                 return "NUMERIC(%(precision)s)" % {'precision' : self.precision}
         else:
-            return "NUMERIC(%(precision)s, %(length)s)" % {'precision': self.precision, 'length' : self.length}
+            return "NUMERIC(%(precision)s, %(scale)s)" % {'precision': self.precision, 'scale' : self.scale}
 
 class SybaseFloat(sqltypes.FLOAT, SybaseNumeric):
-    def __init__(self, precision = 10, asdecimal = False, length = 2, **kwargs):
+    def __init__(self, precision = 10, asdecimal = False, scale = 2, **kwargs):
         super(sqltypes.FLOAT, self).__init__(precision, asdecimal, **kwargs)
-        self.length = length
+        self.scale = scale
 
     def get_col_spec(self):
         # if asdecimal is True, handle same way as SybaseNumeric
@@ -350,16 +350,6 @@ class SybaseUniqueIdentifier(sqltypes.TypeEngine):
     def get_col_spec(self):
         return "UNIQUEIDENTIFIER"
 
-def descriptor():
-    return {'name':'sybase',
-    'description':'SybaseSQL',
-    'arguments':[
-        ('user',"Database Username",None),
-        ('password',"Database Password",None),
-        ('db',"Database Name",None),
-        ('host',"Hostname", None),
-    ]}
-
 class SybaseSQLExecutionContext(default.DefaultExecutionContext):
     pass
 
@@ -460,11 +450,13 @@ class SybaseSQLDialect(default.DefaultDialect):
         'java serialization' : SybaseTypeError,
     }
 
+    name = 'sybase'
     # Sybase backend peculiarities
     supports_unicode_statements = False
     supports_sane_rowcount = False
     supports_sane_multi_rowcount = False
-
+    execution_ctx_cls = SybaseSQLExecutionContext
+    
     def __new__(cls, dbapi=None, *args, **kwargs):
         if cls != SybaseSQLDialect:
             return super(SybaseSQLDialect, cls).__new__(cls, *args, **kwargs)
@@ -487,7 +479,7 @@ class SybaseSQLDialect(default.DefaultDialect):
                 dialect_cls = dialect_mapping[module_name]
                 return dialect_cls.import_dbapi()
             except KeyError:
-                raise exceptions.InvalidRequestError("Unsupported SybaseSQL module '%s' requested (must be " + " or ".join([x for x in dialect_mapping.keys()]) + ")" % module_name)
+                raise exc.InvalidRequestError("Unsupported SybaseSQL module '%s' requested (must be " + " or ".join([x for x in dialect_mapping.keys()]) + ")" % module_name)
         else:
             for dialect_cls in dialect_mapping.values():
                 try:
@@ -498,9 +490,6 @@ class SybaseSQLDialect(default.DefaultDialect):
                 raise ImportError('No DBAPI module detected for SybaseSQL - please install mxodbc')
     dbapi = classmethod(dbapi)
 
-    def create_execution_context(self, *args, **kwargs):
-        return SybaseSQLExecutionContext(self, *args, **kwargs)
-
     def type_descriptor(self, typeobj):
         newobj = sqltypes.adapt_type(typeobj, self.colspecs)
         return newobj
@@ -527,7 +516,7 @@ class SybaseSQLDialect(default.DefaultDialect):
             self.context.rowcount = c.rowcount
             c.DBPROP_COMMITPRESERVE = "Y"
         except Exception, e:
-            raise exceptions.DBAPIError.instance(statement, parameters, e)
+            raise exc.DBAPIError.instance(statement, parameters, e)
 
     def table_names(self, connection, schema):
         """Ignore the schema and the charset for now."""
@@ -597,9 +586,9 @@ class SybaseSQLDialect(default.DefaultDialect):
                               (type, name))
                     coltype = sqltypes.NULLTYPE
                 coltype = coltype(*args)
-            colargs= []
+            colargs = []
             if default is not None:
-                colargs.append(schema.PassiveDefault(sql.text(default)))
+                colargs.append(schema.DefaultClause(sql.text(default)))
 
             # any sequences ?
             col = schema.Column(name, coltype, nullable=nullable, primary_key=primary_key, *colargs)
@@ -624,19 +613,21 @@ class SybaseSQLDialect(default.DefaultDialect):
                 row[0], row[1], row[2], row[3],
             )
             if not primary_table in foreignKeys.keys():
-                foreignKeys[primary_table] = [['%s'%(foreign_column)], ['%s.%s'%(primary_table,primary_column)]]
+                foreignKeys[primary_table] = [['%s' % (foreign_column)], ['%s.%s'%(primary_table, primary_column)]]
             else:
                 foreignKeys[primary_table][0].append('%s'%(foreign_column))
-                foreignKeys[primary_table][1].append('%s.%s'%(primary_table,primary_column))
+                foreignKeys[primary_table][1].append('%s.%s'%(primary_table, primary_column))
         for primary_table in foreignKeys.keys():
             #table.append_constraint(schema.ForeignKeyConstraint(['%s.%s'%(foreign_table, foreign_column)], ['%s.%s'%(primary_table,primary_column)]))
-            table.append_constraint(schema.ForeignKeyConstraint(foreignKeys[primary_table][0], foreignKeys[primary_table][1]))
+            table.append_constraint(schema.ForeignKeyConstraint(foreignKeys[primary_table][0], foreignKeys[primary_table][1], link_to_name=True))
 
         if not found_table:
-            raise exceptions.NoSuchTableError(table.name)
+            raise exc.NoSuchTableError(table.name)
 
 
 class SybaseSQLDialect_mxodbc(SybaseSQLDialect):
+    execution_ctx_cls = SybaseSQLExecutionContext_mxodbc
+    
     def __init__(self, **params):
         super(SybaseSQLDialect_mxodbc, self).__init__(**params)
 
@@ -665,9 +656,6 @@ class SybaseSQLDialect_mxodbc(SybaseSQLDialect):
         #return True
         return False
 
-    def create_execution_context(self, *args, **kwargs):
-        return SybaseSQLExecutionContext_mxodbc(self, *args, **kwargs)
-
     def do_execute(self, cursor, statement, parameters, context=None, **kwargs):
         super(SybaseSQLDialect_mxodbc, self).do_execute(cursor, statement, parameters, context=context, **kwargs)
 
@@ -684,6 +672,8 @@ class SybaseSQLDialect_mxodbc(SybaseSQLDialect):
 
 
 class SybaseSQLDialect_pyodbc(SybaseSQLDialect):
+    execution_ctx_cls = SybaseSQLExecutionContext_pyodbc
+    
     def __init__(self, **params):
         super(SybaseSQLDialect_pyodbc, self).__init__(**params)
         self.dbapi_type_map = {'getdate' : SybaseDate_pyodbc()}
@@ -710,9 +700,6 @@ class SybaseSQLDialect_pyodbc(SybaseSQLDialect):
         #return True
         return False
 
-    def create_execution_context(self, *args, **kwargs):
-        return SybaseSQLExecutionContext_pyodbc(self, *args, **kwargs)
-
     def do_execute(self, cursor, statement, parameters, context=None, **kwargs):
         super(SybaseSQLDialect_pyodbc, self).do_execute(cursor, statement, parameters, context=context, **kwargs)
 
@@ -746,10 +733,18 @@ class SybaseSQLCompiler(compiler.DefaultCompiler):
         sql_operators.mod: lambda x, y: "MOD(%s, %s)" % (x, y),
     })
 
+    extract_map = compiler.DefaultCompiler.extract_map.copy()
+    extract_map.update ({
+        'doy': 'dayofyear',
+        'dow': 'weekday',
+        'milliseconds': 'millisecond'
+    })
+
+
     def bindparam_string(self, name):
         res = super(SybaseSQLCompiler, self).bindparam_string(name)
         if name.lower().startswith('literal'):
-            res = 'STRING(%s)'%res
+            res = 'STRING(%s)' % res
         return res
 
     def get_select_precolumns(self, select):
@@ -780,7 +775,7 @@ class SybaseSQLCompiler(compiler.DefaultCompiler):
             return super(SybaseSQLCompiler, self).visit_binary(binary)
 
     def label_select_column(self, select, column, asfrom):
-        if isinstance(column, expression._Function):
+        if isinstance(column, expression.Function):
             return column.label(None)
         else:
             return super(SybaseSQLCompiler, self).label_select_column(select, column, asfrom)
@@ -799,6 +794,10 @@ class SybaseSQLCompiler(compiler.DefaultCompiler):
             res = "CAST(%s AS %s)" % (res, self.process(cast.typeclause))
         return res
 
+    def visit_extract(self, extract):
+        field = self.extract_map.get(extract.field, extract.field)
+        return 'DATEPART("%s", %s)' % (field, self.process(extract.expr))
+
     def for_update_clause(self, select):
         # "FOR UPDATE" is only allowed on "DECLARE CURSOR" which SQLAlchemy doesn't use
         return ''
@@ -807,7 +806,7 @@ class SybaseSQLCompiler(compiler.DefaultCompiler):
         order_by = self.process(select._order_by_clause)
 
         # SybaseSQL only allows ORDER BY in subqueries if there is a LIMIT
-        if order_by and (not self.is_subquery(select) or select._limit):
+        if order_by and (not self.is_subquery() or select._limit):
             return " ORDER BY " + order_by
         else:
             return ""
@@ -828,7 +827,7 @@ class SybaseSQLSchemaGenerator(compiler.SchemaGenerator):
             #colspec += " numeric(30,0) IDENTITY"
             colspec += " Integer IDENTITY"
         else:
-            colspec += " " + column.type.dialect_impl(self.dialect, _for_ddl=column).get_col_spec()
+            colspec += " " + column.type.dialect_impl(self.dialect).get_col_spec()
 
         if not column.nullable:
             colspec += " NOT NULL"
@@ -844,7 +843,7 @@ class SybaseSQLSchemaDropper(compiler.SchemaDropper):
     def visit_index(self, index):
         self.append("\nDROP INDEX %s.%s" % (
             self.preparer.quote_identifier(index.table.name),
-            self.preparer.quote(index, self._validate_identifier(index.name, False))
+            self.preparer.quote(self._validate_identifier(index.name, False), index.quote)
             ))
         self.execute()
 
