@@ -28,11 +28,11 @@ and return values to events::
 The second approach is a bundle of targeted decorators that wrap appropriate
 append and remove notifiers around the mutation methods present in the
 standard Python ``list``, ``set`` and ``dict`` interfaces.  These could be
-specified in terms of generic decorator recipes, but are instead hand-tooled for
-increased efficiency.   The targeted decorators occasionally implement
-adapter-like behavior, such as mapping bulk-set methods (``extend``, ``update``,
-``__setslice__``, etc.) into the series of atomic mutation events that the ORM
-requires.
+specified in terms of generic decorator recipes, but are instead hand-tooled
+for increased efficiency.  The targeted decorators occasionally implement
+adapter-like behavior, such as mapping bulk-set methods (``extend``,
+``update``, ``__setslice__``, etc.) into the series of atomic mutation events
+that the ORM requires.
 
 The targeted decorators are used internally for automatic instrumentation of
 entity collection classes.  Every collection class goes through a
@@ -93,22 +93,27 @@ explicit control over triggering append and remove events.
 
 The owning object and InstrumentedCollectionAttribute are also reachable
 through the adapter, allowing for some very sophisticated behavior.
+
 """
 
 import copy
 import inspect
-import sets
+import operator
 import sys
 import weakref
 
-from sqlalchemy import exceptions, schema, util as sautil
-from sqlalchemy.util import attrgetter, Set
+import sqlalchemy.exceptions as sa_exc
+from sqlalchemy.sql import expression
+from sqlalchemy import schema, util
 
 
 __all__ = ['collection', 'collection_adapter',
            'mapped_collection', 'column_mapped_collection',
            'attribute_mapped_collection']
 
+__instrumentation_mutex = util.threading.Lock()
+
+
 def column_mapped_collection(mapping_spec):
     """A dictionary-based collection type with column-based keying.
 
@@ -119,25 +124,24 @@ def column_mapped_collection(mapping_spec):
     can not, for example, map on foreign key values if those key values will
     change during the session, i.e. from None to a database-assigned integer
     after a session flush.
+
     """
+    from sqlalchemy.orm.util import _state_mapper
+    from sqlalchemy.orm.attributes import instance_state
 
-    from sqlalchemy.orm import object_mapper
-
-    if isinstance(mapping_spec, schema.Column):
+    cols = [expression._no_literals(q) for q in util.to_list(mapping_spec)]
+    if len(cols) == 1:
         def keyfunc(value):
-            m = object_mapper(value)
-            return m._get_attr_by_column(value, mapping_spec)
+            state = instance_state(value)
+            m = _state_mapper(state)
+            return m._get_state_attr_by_column(state, cols[0])
     else:
-        cols = []
-        for c in mapping_spec:
-            if not isinstance(c, schema.Column):
-                raise exceptions.ArgumentError(
-                    "mapping_spec tuple may only contain columns")
-            cols.append(c)
         mapping_spec = tuple(cols)
         def keyfunc(value):
-            m = object_mapper(value)
-            return tuple([m._get_attr_by_column(value, c) for c in mapping_spec])
+            state = instance_state(value)
+            m = _state_mapper(state)
+            return tuple(m._get_state_attr_by_column(state, c)
+                         for c in mapping_spec)
     return lambda: MappedCollection(keyfunc)
 
 def attribute_mapped_collection(attr_name):
@@ -150,9 +154,9 @@ def attribute_mapped_collection(attr_name):
     can not, for example, map on foreign key values if those key values will
     change during the session, i.e. from None to a database-assigned integer
     after a session flush.
-    """
 
-    return lambda: MappedCollection(attrgetter(attr_name))
+    """
+    return lambda: MappedCollection(operator.attrgetter(attr_name))
 
 
 def mapped_collection(keyfunc):
@@ -165,8 +169,8 @@ def mapped_collection(keyfunc):
     can not, for example, map on foreign key values if those key values will
     change during the session, i.e. from None to a database-assigned integer
     after a session flush.
-    """
 
+    """
     return lambda: MappedCollection(keyfunc)
 
 class collection(object):
@@ -193,12 +197,13 @@ class collection(object):
     Decorators can be specified in long-hand for Python 2.3, or with
     the class-level dict attribute '__instrumentation__'- see the source
     for details.
-    """
 
+    """
     # Bundled as a class solely for ease of use: packaging, doc strings,
     # importability.
 
-    def appender(cls, fn):
+    @staticmethod
+    def appender(fn):
         """Tag the method as the collection appender.
 
         The appender method is called with one positional argument: the value
@@ -236,13 +241,13 @@ class collection(object):
         If the appender method is internally instrumented, you must also
         receive the keyword argument '_sa_initiator' and ensure its
         promulgation to collection events.
-        """
 
+        """
         setattr(fn, '_sa_instrument_role', 'appender')
         return fn
-    appender = classmethod(appender)
 
-    def remover(cls, fn):
+    @staticmethod
+    def remover(fn):
         """Tag the method as the collection remover.
 
         The remover method is called with one positional argument: the value
@@ -263,13 +268,13 @@ class collection(object):
         If the remove method is internally instrumented, you must also
         receive the keyword argument '_sa_initiator' and ensure its
         promulgation to collection events.
-        """
 
+        """
         setattr(fn, '_sa_instrument_role', 'remover')
         return fn
-    remover = classmethod(remover)
 
-    def iterator(cls, fn):
+    @staticmethod
+    def iterator(fn):
         """Tag the method as the collection remover.
 
         The iterator method is called with no arguments.  It is expected to
@@ -277,13 +282,13 @@ class collection(object):
 
             @collection.iterator
             def __iter__(self): ...
-        """
 
+        """
         setattr(fn, '_sa_instrument_role', 'iterator')
         return fn
-    iterator = classmethod(iterator)
 
-    def internally_instrumented(cls, fn):
+    @staticmethod
+    def internally_instrumented(fn):
         """Tag the method as instrumented.
 
         This tag will prevent any decoration from being applied to the method.
@@ -297,13 +302,13 @@ class collection(object):
             # never be called, unless:
             @collection.internally_instrumented
             def extend(self, items): ...
-        """
 
+        """
         setattr(fn, '_sa_instrumented', True)
         return fn
-    internally_instrumented = classmethod(internally_instrumented)
 
-    def on_link(cls, fn):
+    @staticmethod
+    def on_link(fn):
         """Tag the method as a the "linked to attribute" event handler.
 
         This optional event handler will be called when the collection class
@@ -311,13 +316,13 @@ class collection(object):
         invoked immediately after the '_sa_adapter' property is set on
         the instance.  A single argument is passed: the collection adapter
         that has been linked, or None if unlinking.
-        """
 
+        """
         setattr(fn, '_sa_instrument_role', 'on_link')
         return fn
-    on_link = classmethod(on_link)
 
-    def converter(cls, fn):
+    @staticmethod
+    def converter(fn):
         """Tag the method as the collection converter.
 
         This optional method will be called when a collection is being
@@ -344,33 +349,34 @@ class collection(object):
         Supply an implementation of this method if you want to expand the
         range of possible types that can be assigned in bulk or perform
         validation on the values about to be assigned.
-        """
 
+        """
         setattr(fn, '_sa_instrument_role', 'converter')
         return fn
-    converter = classmethod(converter)
 
-    def adds(cls, arg):
+    @staticmethod
+    def adds(arg):
         """Mark the method as adding an entity to the collection.
 
-        Adds "add to collection" handling to the method.  The decorator argument
-        indicates which method argument holds the SQLAlchemy-relevant value.
-        Arguments can be specified positionally (i.e. integer) or by name::
+        Adds "add to collection" handling to the method.  The decorator
+        argument indicates which method argument holds the SQLAlchemy-relevant
+        value.  Arguments can be specified positionally (i.e. integer) or by
+        name::
 
             @collection.adds(1)
             def push(self, item): ...
 
             @collection.adds('entity')
             def do_stuff(self, thing, entity=None): ...
-        """
 
+        """
         def decorator(fn):
             setattr(fn, '_sa_instrument_before', ('fire_append_event', arg))
             return fn
         return decorator
-    adds = classmethod(adds)
 
-    def replaces(cls, arg):
+    @staticmethod
+    def replaces(arg):
         """Mark the method as replacing an entity in the collection.
 
         Adds "add to collection" and "remove from collection" handling to
@@ -382,16 +388,16 @@ class collection(object):
 
             @collection.replaces(2)
             def __setitem__(self, index, item): ...
-        """
 
+        """
         def decorator(fn):
             setattr(fn, '_sa_instrument_before', ('fire_append_event', arg))
             setattr(fn, '_sa_instrument_after', 'fire_remove_event')
             return fn
         return decorator
-    replaces = classmethod(replaces)
 
-    def removes(cls, arg):
+    @staticmethod
+    def removes(arg):
         """Mark the method as removing an entity in the collection.
 
         Adds "remove from collection" handling to the method.  The decorator
@@ -404,15 +410,15 @@ class collection(object):
 
         For methods where the value to remove is not known at call-time, use
         collection.removes_return.
-        """
 
+        """
         def decorator(fn):
             setattr(fn, '_sa_instrument_before', ('fire_remove_event', arg))
             return fn
         return decorator
-    removes = classmethod(removes)
 
-    def removes_return(cls):
+    @staticmethod
+    def removes_return():
         """Mark the method as removing an entity in the collection.
 
         Adds "remove from collection" handling to the method.  The return value
@@ -424,20 +430,18 @@ class collection(object):
 
         For methods where the value to remove is known at call-time, use
         collection.remove.
-        """
 
+        """
         def decorator(fn):
             setattr(fn, '_sa_instrument_after', 'fire_remove_event')
             return fn
         return decorator
-    removes_return = classmethod(removes_return)
 
 
 # public instrumentation interface for 'internally instrumented'
 # implementations
 def collection_adapter(collection):
     """Fetch the CollectionAdapter for a collection."""
-
     return getattr(collection, '_sa_adapter', None)
 
 def collection_iter(collection):
@@ -445,8 +449,8 @@ def collection_iter(collection):
 
     If the collection is an ORM collection, it need not be attached to an
     object to be iterable.
-    """
 
+    """
     try:
         return getattr(collection, '_sa_iterator',
                        getattr(collection, '__iter__'))()
@@ -464,10 +468,11 @@ class CollectionAdapter(object):
 
     The ORM uses an CollectionAdapter exclusively for interaction with
     entity collections.
-    """
 
+    """
     def __init__(self, attr, owner_state, data):
         self.attr = attr
+        # TODO: figure out what this being a weakref buys us
         self._data = weakref.ref(data)
         self.owner_state = owner_state
         self.link_to_self(data)
@@ -477,14 +482,12 @@ class CollectionAdapter(object):
 
     def link_to_self(self, data):
         """Link a collection to this adapter, and fire a link event."""
-
         setattr(data, '_sa_adapter', self)
         if hasattr(data, '_sa_on_link'):
             getattr(data, '_sa_on_link')(self)
 
     def unlink(self, data):
         """Unlink a collection from any adapter, and fire a link event."""
-
         setattr(data, '_sa_adapter', None)
         if hasattr(data, '_sa_on_link'):
             getattr(data, '_sa_on_link')(None)
@@ -501,14 +504,14 @@ class CollectionAdapter(object):
 
         If a converter implementation is not supplied on the collection,
         a default duck-typing-based implementation is used.
-        """
 
+        """
         converter = getattr(self._data(), '_sa_converter', None)
         if converter is not None:
             return converter(obj)
 
-        setting_type = sautil.duck_type_collection(obj)
-        receiving_type = sautil.duck_type_collection(self._data())
+        setting_type = util.duck_type_collection(obj)
+        receiving_type = util.duck_type_collection(self._data())
 
         if obj is None or setting_type != receiving_type:
             given = obj is None and 'None' or obj.__class__.__name__
@@ -521,7 +524,8 @@ class CollectionAdapter(object):
                 "Incompatible collection type: %s is not %s-like" % (
                 given, wanted))
 
-        # If the object is an adapted collection, return the (iterable) adapter.
+        # If the object is an adapted collection, return the (iterable)
+        # adapter.
         if getattr(obj, '_sa_adapter', None) is not None:
             return getattr(obj, '_sa_adapter')
         elif setting_type == dict:
@@ -531,44 +535,36 @@ class CollectionAdapter(object):
 
     def append_with_event(self, item, initiator=None):
         """Add an entity to the collection, firing mutation events."""
-
         getattr(self._data(), '_sa_appender')(item, _sa_initiator=initiator)
 
     def append_without_event(self, item):
         """Add or restore an entity to the collection, firing no events."""
-
         getattr(self._data(), '_sa_appender')(item, _sa_initiator=False)
 
     def remove_with_event(self, item, initiator=None):
         """Remove an entity from the collection, firing mutation events."""
-
         getattr(self._data(), '_sa_remover')(item, _sa_initiator=initiator)
 
     def remove_without_event(self, item):
         """Remove an entity from the collection, firing no events."""
-
         getattr(self._data(), '_sa_remover')(item, _sa_initiator=False)
 
     def clear_with_event(self, initiator=None):
         """Empty the collection, firing a mutation event for each entity."""
-
         for item in list(self):
             self.remove_with_event(item, initiator)
 
     def clear_without_event(self):
         """Empty the collection, firing no events."""
-
         for item in list(self):
             self.remove_without_event(item)
 
     def __iter__(self):
         """Iterate over entities in the collection."""
-
         return getattr(self._data(), '_sa_iterator')()
 
     def __len__(self):
         """Count entities in the collection."""
-
         return len(list(getattr(self._data(), '_sa_iterator')()))
 
     def __nonzero__(self):
@@ -580,10 +576,12 @@ class CollectionAdapter(object):
         Initiator is the InstrumentedAttribute that initiated the membership
         mutation, and should be left as None unless you are passing along
         an initiator value from a chained operation.
-        """
 
+        """
         if initiator is not False and item is not None:
-            self.attr.fire_append_event(self.owner_state, item, initiator)
+            return self.attr.fire_append_event(self.owner_state, self.owner_state.dict, item, initiator)
+        else:
+            return item
 
     def fire_remove_event(self, item, initiator=None):
         """Notify that a entity has been removed from the collection.
@@ -591,24 +589,24 @@ class CollectionAdapter(object):
         Initiator is the InstrumentedAttribute that initiated the membership
         mutation, and should be left as None unless you are passing along
         an initiator value from a chained operation.
-        """
 
+        """
         if initiator is not False and item is not None:
-            self.attr.fire_remove_event(self.owner_state, item, initiator)
+            self.attr.fire_remove_event(self.owner_state, self.owner_state.dict, item, initiator)
 
     def fire_pre_remove_event(self, initiator=None):
         """Notify that an entity is about to be removed from the collection.
 
         Only called if the entity cannot be removed after calling
         fire_remove_event().
-        """
 
-        self.attr.fire_pre_remove_event(self.owner_state, initiator=initiator)
+        """
+        self.attr.fire_pre_remove_event(self.owner_state, self.owner_state.dict, initiator=initiator)
 
     def __getstate__(self):
-        return { 'key': self.attr.key,
-                 'owner_state': self.owner_state,
-                 'data': self.data }
+        return {'key': self.attr.key,
+                'owner_state': self.owner_state,
+                'data': self.data}
 
     def __setstate__(self, d):
         self.attr = getattr(d['owner_state'].obj().__class__, d['key']).impl
@@ -638,10 +636,10 @@ def bulk_replace(values, existing_adapter, new_adapter):
     if not isinstance(values, list):
         values = list(values)
 
-    idset = sautil.IdentitySet
+    idset = util.IdentitySet
     constants = idset(existing_adapter or ()).intersection(values or ())
     additions = idset(values or ()).difference(constants)
-    removals  = idset(existing_adapter or ()).difference(constants)
+    removals = idset(existing_adapter or ()).difference(constants)
 
     for member in values or ():
         if member in additions:
@@ -653,8 +651,7 @@ def bulk_replace(values, existing_adapter, new_adapter):
         for member in removals:
             existing_adapter.remove_with_event(member)
 
-__instrumentation_mutex = sautil.threading.Lock()
-def _prepare_instrumentation(factory):
+def prepare_instrumentation(factory):
     """Prepare a callable for future use as a collection class factory.
 
     Given a collection class factory (either a type or no-arg callable),
@@ -663,8 +660,8 @@ def _prepare_instrumentation(factory):
 
     This function is responsible for converting collection_class=list
     into the run-time behavior of collection_class=InstrumentedList.
-    """
 
+    """
     # Convert a builtin to 'Instrumented*'
     if factory in __canned_instrumentation:
         factory = __canned_instrumentation[factory]
@@ -694,8 +691,8 @@ def __converting_factory(original_factory):
     Given a collection factory that returns a builtin type (e.g. a list),
     return a wrapped function that converts that type to one of our
     instrumented types.
-    """
 
+    """
     def wrapper():
         collection = original_factory()
         type_ = type(collection)
@@ -704,7 +701,7 @@ def __converting_factory(original_factory):
             # collection
             return __canned_instrumentation[type_](collection)
         else:
-            raise exceptions.InvalidRequestError(
+            raise sa_exc.InvalidRequestError(
                 "Collection class factories must produce instances of a "
                 "single class.")
     try:
@@ -718,9 +715,9 @@ def __converting_factory(original_factory):
 def _instrument_class(cls):
     """Modify methods in a class and install instrumentation."""
 
-    # FIXME: more formally document this as a decoratorless/Python 2.3
+    # TODO: more formally document this as a decoratorless/Python 2.3
     # option for specifying instrumentation.  (likely doc'd here in code only,
-    # not in online docs.)
+    # not in online docs.)  Useful for C types too.
     #
     # __instrumentation__ = {
     #   'rolename': 'methodname', # ...
@@ -737,11 +734,11 @@ def _instrument_class(cls):
     # types is transformed into one of our trivial subclasses
     # (e.g. InstrumentedList).  Catch anything else that sneaks in here...
     if cls.__module__ == '__builtin__':
-        raise exceptions.ArgumentError(
+        raise sa_exc.ArgumentError(
             "Can not instrument a built-in type. Use a "
             "subclass, even a trivial one.")
 
-    collection_type = sautil.duck_type_collection(cls)
+    collection_type = util.duck_type_collection(cls)
     if collection_type in __interfaces:
         roles = __interfaces[collection_type].copy()
         decorators = roles.pop('_decorators', {})
@@ -755,7 +752,7 @@ def _instrument_class(cls):
 
     for name in dir(cls):
         method = getattr(cls, name, None)
-        if not callable(method):
+        if not util.callable(method):
             continue
 
         # note role declarations
@@ -784,13 +781,14 @@ def _instrument_class(cls):
     # apply ABC auto-decoration to methods that need it
     for method, decorator in decorators.items():
         fn = getattr(cls, method, None)
-        if fn and method not in methods and not hasattr(fn, '_sa_instrumented'):
+        if (fn and method not in methods and
+            not hasattr(fn, '_sa_instrumented')):
             setattr(cls, method, decorator(fn))
 
     # ensure all roles are present, and apply implicit instrumentation if
     # needed
     if 'appender' not in roles or not hasattr(cls, roles['appender']):
-        raise exceptions.ArgumentError(
+        raise sa_exc.ArgumentError(
             "Type %s must elect an appender method to be "
             "a collection class" % cls.__name__)
     elif (roles['appender'] not in methods and
@@ -798,7 +796,7 @@ def _instrument_class(cls):
         methods[roles['appender']] = ('fire_append_event', 1, None)
 
     if 'remover' not in roles or not hasattr(cls, roles['remover']):
-        raise exceptions.ArgumentError(
+        raise sa_exc.ArgumentError(
             "Type %s must elect a remover method to be "
             "a collection class" % cls.__name__)
     elif (roles['remover'] not in methods and
@@ -806,7 +804,7 @@ def _instrument_class(cls):
         methods[roles['remover']] = ('fire_remove_event', 1, None)
 
     if 'iterator' not in roles or not hasattr(cls, roles['iterator']):
-        raise exceptions.ArgumentError(
+        raise sa_exc.ArgumentError(
             "Type %s must elect an iterator method to be "
             "a collection class" % cls.__name__)
 
@@ -824,10 +822,9 @@ def _instrument_class(cls):
 
 def _instrument_membership_mutator(method, before, argument, after):
     """Route method args and/or return value through the collection adapter."""
-
     # This isn't smart enough to handle @adds(1) for 'def fn(self, (a, b))'
     if before:
-        fn_args = list(sautil.flatten_iterator(inspect.getargspec(method)[0]))
+        fn_args = list(util.flatten_iterator(inspect.getargspec(method)[0]))
         if type(argument) is int:
             pos_arg = argument
             named_arg = len(fn_args) > argument and fn_args[argument] or None
@@ -843,7 +840,7 @@ def _instrument_membership_mutator(method, before, argument, after):
         if before:
             if pos_arg is None:
                 if named_arg not in kw:
-                    raise exceptions.ArgumentError(
+                    raise sa_exc.ArgumentError(
                         "Missing argument %s" % argument)
                 value = kw[named_arg]
             else:
@@ -852,7 +849,7 @@ def _instrument_membership_mutator(method, before, argument, after):
                 elif named_arg in kw:
                     value = kw[named_arg]
                 else:
-                    raise exceptions.ArgumentError(
+                    raise sa_exc.ArgumentError(
                         "Missing argument %s" % argument)
 
         initiator = kw.pop('_sa_initiator', None)
@@ -885,11 +882,11 @@ def __set(collection, item, _sa_initiator=None):
     if _sa_initiator is not False and item is not None:
         executor = getattr(collection, '_sa_adapter', None)
         if executor:
-            getattr(executor, 'fire_append_event')(item, _sa_initiator)
-
+            item = getattr(executor, 'fire_append_event')(item, _sa_initiator)
+    return item
+    
 def __del(collection, item, _sa_initiator=None):
     """Run del events, may eventually be inlined into decorators."""
-
     if _sa_initiator is not False and item is not None:
         executor = getattr(collection, '_sa_adapter', None)
         if executor:
@@ -897,14 +894,12 @@ def __del(collection, item, _sa_initiator=None):
 
 def __before_delete(collection, _sa_initiator=None):
     """Special method to run 'commit existing value' methods"""
-
     executor = getattr(collection, '_sa_adapter', None)
     if executor:
         getattr(executor, 'fire_pre_remove_event')(_sa_initiator)
 
 def _list_decorators():
-    """Hand-turned instrumentation wrappers that can decorate any list-like
-    class."""
+    """Tailored instrumentation wrappers for any list-like class."""
 
     def _tidy(fn):
         setattr(fn, '_sa_instrumented', True)
@@ -912,13 +907,7 @@ def _list_decorators():
 
     def append(fn):
         def append(self, item, _sa_initiator=None):
-            # FIXME: example of fully inlining __set and adapter.fire
-            # for critical path
-            if _sa_initiator is not False and item is not None:
-                executor = getattr(self, '_sa_adapter', None)
-                if executor:
-                    executor.attr.fire_append_event(executor.owner_state,
-                                                    item, _sa_initiator)
+            item = __set(self, item, _sa_initiator)
             fn(self, item)
         _tidy(append)
         return append
@@ -934,7 +923,7 @@ def _list_decorators():
 
     def insert(fn):
         def insert(self, index, value):
-            __set(self, value)
+            value = __set(self, value)
             fn(self, index, value)
         _tidy(insert)
         return insert
@@ -945,7 +934,7 @@ def _list_decorators():
                 existing = self[index]
                 if existing is not None:
                     __del(self, existing)
-                __set(self, value)
+                value = __set(self, value)
                 fn(self, index, value)
             else:
                 # slice assignment requires __delitem__, insert, __len__
@@ -995,8 +984,7 @@ def _list_decorators():
         def __setslice__(self, start, end, values):
             for value in self[start:end]:
                 __del(self, value)
-            for value in values:
-                __set(self, value)
+            values = [__set(self, value) for value in values]
             fn(self, start, end, values)
         _tidy(__setslice__)
         return __setslice__
@@ -1045,20 +1033,19 @@ def _list_decorators():
     return l
 
 def _dict_decorators():
-    """Hand-turned instrumentation wrappers that can decorate any dict-like
-    mapping class."""
+    """Tailored instrumentation wrappers for any dict-like mapping class."""
 
     def _tidy(fn):
         setattr(fn, '_sa_instrumented', True)
         fn.__doc__ = getattr(getattr(dict, fn.__name__), '__doc__')
 
-    Unspecified=sautil.symbol('Unspecified')
+    Unspecified = util.symbol('Unspecified')
 
     def __setitem__(fn):
         def __setitem__(self, key, value, _sa_initiator=None):
             if key in self:
                 __del(self, self[key], _sa_initiator)
-            __set(self, value, _sa_initiator)
+            value = __set(self, value, _sa_initiator)
             fn(self, key, value)
         _tidy(__setitem__)
         return __setitem__
@@ -1123,7 +1110,8 @@ def _dict_decorators():
                 if __other is not Unspecified:
                     if hasattr(__other, 'keys'):
                         for key in __other.keys():
-                            if key not in self or self[key] is not __other[key]:
+                            if (key not in self or
+                                self[key] is not __other[key]):
                                 self[key] = __other[key]
                     else:
                         for key, value in __other:
@@ -1140,11 +1128,11 @@ def _dict_decorators():
     l.pop('Unspecified')
     return l
 
-
-try:
+if util.py3k:
+    _set_binop_bases = (set, frozenset)
+else:
+    import sets
     _set_binop_bases = (set, frozenset, sets.BaseSet)
-except NameError:
-    _set_binop_bases = (sets.BaseSet,)
 
 def _set_binops_check_strict(self, obj):
     """Allow only set, frozenset and self.__class__-derived objects in binops."""
@@ -1153,23 +1141,22 @@ def _set_binops_check_strict(self, obj):
 def _set_binops_check_loose(self, obj):
     """Allow anything set-like to participate in set binops."""
     return (isinstance(obj, _set_binop_bases + (self.__class__,)) or
-            sautil.duck_type_collection(obj) == sautil.Set)
+            util.duck_type_collection(obj) == set)
 
 
 def _set_decorators():
-    """Hand-turned instrumentation wrappers that can decorate any set-like
-    sequence class."""
+    """Tailored instrumentation wrappers for any set-like class."""
 
     def _tidy(fn):
         setattr(fn, '_sa_instrumented', True)
-        fn.__doc__ = getattr(getattr(Set, fn.__name__), '__doc__')
+        fn.__doc__ = getattr(getattr(set, fn.__name__), '__doc__')
 
-    Unspecified=sautil.symbol('Unspecified')
+    Unspecified = util.symbol('Unspecified')
 
     def add(fn):
         def add(self, value, _sa_initiator=None):
             if value not in self:
-                __set(self, value, _sa_initiator)
+                value = __set(self, value, _sa_initiator)
             # testlib.pragma exempt:__hash__
             fn(self, value)
         _tidy(add)
@@ -1255,7 +1242,7 @@ def _set_decorators():
 
     def intersection_update(fn):
         def intersection_update(self, other):
-            want, have = self.intersection(other), Set(self)
+            want, have = self.intersection(other), set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1269,7 +1256,7 @@ def _set_decorators():
         def __iand__(self, other):
             if not _set_binops_check_strict(self, other):
                 return NotImplemented
-            want, have = self.intersection(other), Set(self)
+            want, have = self.intersection(other), set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1282,7 +1269,7 @@ def _set_decorators():
 
     def symmetric_difference_update(fn):
         def symmetric_difference_update(self, other):
-            want, have = self.symmetric_difference(other), Set(self)
+            want, have = self.symmetric_difference(other), set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1296,7 +1283,7 @@ def _set_decorators():
         def __ixor__(self, other):
             if not _set_binops_check_strict(self, other):
                 return NotImplemented
-            want, have = self.symmetric_difference(other), Set(self)
+            want, have = self.symmetric_difference(other), set(self)
             remove, add = have - want, want - have
 
             for item in remove:
@@ -1321,8 +1308,8 @@ class InstrumentedList(list):
        'remover': 'remove',
        'iterator': '__iter__', }
 
-class InstrumentedSet(Set):
-    """An instrumented version of the built-in set (or Set)."""
+class InstrumentedSet(set):
+    """An instrumented version of the built-in set."""
 
     __instrumentation__ = {
        'appender': 'add',
@@ -1337,27 +1324,26 @@ class InstrumentedDict(dict):
 
 __canned_instrumentation = {
     list: InstrumentedList,
-    Set: InstrumentedSet,
+    set: InstrumentedSet,
     dict: InstrumentedDict,
     }
 
 __interfaces = {
-    list: { 'appender': 'append',
-            'remover':  'remove',
-            'iterator': '__iter__',
-            '_decorators': _list_decorators(), },
-    Set: { 'appender': 'add',
+    list: {'appender': 'append',
            'remover': 'remove',
            'iterator': '__iter__',
-           '_decorators': _set_decorators(), },
+           '_decorators': _list_decorators(), },
+    set: {'appender': 'add',
+          'remover': 'remove',
+          'iterator': '__iter__',
+          '_decorators': _set_decorators(), },
     # decorators are required for dicts and object collections.
-    dict: { 'iterator': 'itervalues',
-            '_decorators': _dict_decorators(), },
+    dict: {'iterator': 'itervalues',
+           '_decorators': _dict_decorators(), },
     # < 0.4 compatible naming, deprecated- use decorators instead.
-    None: { }
+    None: {}
     }
 
-
 class MappedCollection(dict):
     """A basic dictionary-based collection class.
 
@@ -1365,6 +1351,7 @@ class MappedCollection(dict):
     ``set`` and ``remove`` are implemented in terms of a keying function: any
     callable that takes an object and returns an object for use as a dictionary
     key.
+
     """
 
     def __init__(self, keyfunc):
@@ -1374,16 +1361,17 @@ class MappedCollection(dict):
         returns an object for use as a dictionary key.
 
         The keyfunc will be called every time the ORM needs to add a member by
-        value-only (such as when loading instances from the database) or remove
-        a member.  The usual cautions about dictionary keying apply-
+        value-only (such as when loading instances from the database) or
+        remove a member.  The usual cautions about dictionary keying apply-
         ``keyfunc(object)`` should return the same output for the life of the
         collection.  Keying based on mutable properties can result in
         unreachable instances "lost" in the collection.
+
         """
         self.keyfunc = keyfunc
 
     def set(self, value, _sa_initiator=None):
-        """Add an item to the collection, with a key provided by this instance's keyfunc."""
+        """Add an item by value, consulting the keyfunc for the key."""
 
         key = self.keyfunc(value)
         self.__setitem__(key, value, _sa_initiator)
@@ -1391,13 +1379,13 @@ class MappedCollection(dict):
     set = collection.appender(set)
 
     def remove(self, value, _sa_initiator=None):
-        """Remove an item from the collection by value, consulting this instance's keyfunc for the key."""
+        """Remove an item by value, consulting the keyfunc for the key."""
 
         key = self.keyfunc(value)
         # Let self[key] raise if key is not in this collection
         # testlib.pragma exempt:__ne__
         if self[key] != value:
-            raise exceptions.InvalidRequestError(
+            raise sa_exc.InvalidRequestError(
                 "Can not remove '%s': collection holds '%s' for key '%s'. "
                 "Possible cause: is the MappedCollection key function "
                 "based on mutable properties or properties that only obtain "
@@ -1418,9 +1406,9 @@ class MappedCollection(dict):
         Raises a TypeError if the key in any (key, value) pair in the dictlike
         object does not match the key that this collection's keyfunc would
         have assigned for that value.
-        """
 
-        for incoming_key, value in sautil.dictlike_iteritems(dictlike):
+        """
+        for incoming_key, value in util.dictlike_iteritems(dictlike):
             new_key = self.keyfunc(value)
             if incoming_key != new_key:
                 raise TypeError(
