@@ -95,6 +95,21 @@ class AdaptTest(TestBase):
         ]:
             assert isinstance(start.dialect_impl(dialect), test), "wanted %r got %r" % (test, start.dialect_impl(dialect))
 
+class TypeAffinityTest(TestBase):
+    def test_type_affinity(self):
+        for t1, t2, comp in [
+            (Integer(), SmallInteger(), True),
+            (Integer(), String(), False),
+            (Integer(), Integer(), True),
+            (Text(), String(), True),
+            (Text(), Unicode(), True),
+            (Binary(), Integer(), False),
+            (Binary(), PickleType(), True),
+            (PickleType(), Binary(), True),
+            (PickleType(), PickleType(), True),
+        ]:
+            eq_(t1._compare_type_affinity(t2), comp, "%s %s" % (t1, t2))
+
 
 
 class UserDefinedTest(TestBase):
@@ -529,6 +544,16 @@ class ExpressionTest(TestBase, AssertsExecutionResults):
         # this one relies upon anonymous labeling to assemble result
         # processing rules on the column.
         assert testing.db.execute(select([expr])).scalar() == -15
+        
+    def test_distinct(self):
+        s = select([distinct(test_table.c.avalue)])
+        eq_(testing.db.execute(s).scalar(), 25)
+
+        s = select([test_table.c.avalue.distinct()])
+        eq_(testing.db.execute(s).scalar(), 25)
+
+        assert distinct(test_table.c.data).type == test_table.c.data.type
+        assert test_table.c.data.distinct().type == test_table.c.data.type
 
 class DateTest(TestBase, AssertsExecutionResults):
     @classmethod
